"""
web2ldapcnf/hosts.py - Host-related options
(c) by Michael Stroeder <michael@stroeder.com>
"""

# Leave this alone
import os,web2ldapcnf

# List of all LDAP hosts to use. These hosts will appear in the
# default select list of the login form.
# A list containing only one host results in a normal input field
# with the host set as default.
ldap_uri_list = [
  'ldap://localhost',
  'ldap://localhost:1389',
  'ldaps://localhost:1636',
  (
    'ldapi://%2ftmp%2fopenldap2/dc=stroeder,dc=com',
    u'Local OpenLDAP 2.0.x server (Unix domain socket)'
  ),
  (
    'ldap://localhost:1390',
    u'Local OpenLDAP 2.1.x server'
  ),
  (
    'ldapi://%2Ftmp%2Fopenldap-REL_ENG_2_1',
    u'Local OpenLDAP 2.1.x server (Unix domain socket)'
  ),
  'ldap://localhost:2389',
  'ldap://localhost:3389',
  'ldap://localhost:4389',
  'ldap://localhost:5389',
  'ldap://localhost:6389/dc=stroeder,dc=com',
  (
    'ldap://ldap.openldap.org',
    u"OpenLDAP demo server"
  ),
  (
    'ldap://www.nldap.com',
    u"Novell eDirectory Services Test Site"
  ),
  'ldap://ldap.surfnet.nl',
  'ldap://ldap.rediris.es',
  'ldap://ldap.uninett.no',
  (
    'ldap://www.trustcenter.de',
    u'LDAP-based PKI repository of TC Trustcenter'
  ),
  (
    'ldap://x500.bund.de/o=Bund,c=DE??one?',
    u'X.500 Directory Informationsverbund Berlin-Bonn'
  ),
  'ldap://db.debian.org',
  'ldap://ldap.nameflow.net',
  'ldap://ldap.itd.umich.edu',
  'ldap://ldap.baylor.edu',
  'ldap://directory.uga.edu',
  'ldap://memberdir.netscape.com',
  'ldap://directory.verisign.com',
  'ldap://x500.umn.edu',
]

########################################################################
# LDAP host(s) with their defaults can be pre-defined as dictionary
# ldap_def = {'host:port':{paramdictionary}}
########################################################################


class Web2LDAPConfig:
  """
  Base class for a web2ldap host-/backend configuration section.
  
  Leave untouched!!!
  """
  def __init__(self,**kwargs):
    self.__dict__.update(kwargs)
  def get(self,name,default=None):
    self.__dict__.get(name,default)

ldap_def = {

  # Overall default values.
  # There's most times no need to adjust the values in this section
  # since they can be overridden by host specific sections (see below).
  '_': Web2LDAPConfig(

    # Search filter template for smart login
    # Use indexed attributes here!
    binddnsearch=r'(|(cn=%s)(uid=%s))',
    # Example for M$ Active Directory
#    binddnsearch=r'(|(sAMAccountName=%s)(userPrincipalName=%s))',

    # Timeout for synchronous LDAP operations (seconds)
    timeout=60,

    # Use StartTLS on connect. It's recommended saying 0 as default
    # since not many servers can handle an unknown extended control properly
    starttls=0,

    # Maximum amount of time in seconds a result is valid in the cache
    cache_timeout=900,
    # Maximum cache size in bytes (per session!)
    cache_maxmem=0,
    # regular expression of subject DNs in client certs
    ssl_valid_dn=r'/.*',
    # regular expression of subject DNs in issuer certs
    ssl_valid_idn=r'/.*',
    sec_reqlevel=0,
    # Attributes explicitly requested while doing read
    # and modify operations
    requested_attrs=[
      # password policy attributes
      'passwordExpirationTime','passwordExpWarned',
      'passwordRetryCount','retryCountResetTime','accountUnlockTime',
      'passwordHistory','passwordAllowChangeTime',
      # Account specifics
      'nsAccountLock','nsLookthroughLimit',
      # ACL attributes
      'aci',
      # counters
      'numSubordinates',
    ],
    # vCard template files
    vcard_template={
      # 'object class':'pathname of vCard template file'
      'person':os.path.join(web2ldapcnf.templates_dir,'vcard_person.txt'),
      'inetOrgPerson':os.path.join(web2ldapcnf.templates_dir,'vcard_person.txt')
    },
    # HTML template files for printing table entries
    print_template={
      # 'object class':'pathname of printable HTML template file'
      'person':os.path.join(web2ldapcnf.templates_dir,'print_person.html'),
      'organization':os.path.join(web2ldapcnf.templates_dir,'print_organization.html'),
      'organizationalUnit':os.path.join(web2ldapcnf.templates_dir,'print_organizationalUnit.html'),
    },
    # HTML template file for Basic Search form
    searchform_template= os.path.join(web2ldapcnf.templates_dir,'searchform_Base.html'),
    # HTML template file for Login form
    login_template= os.path.join(web2ldapcnf.templates_dir,'login.html'),
    # Attributes which should be present in attribute select list of advanced search form
    search_attrs=['cn','mail','o','ou','telephoneNumber','homePhone','l','description'],
    # There are some situations where this client just wants to get the
    # attributes of an entry and not the data itself for saving bandwidth.
    # However some LDAP hosts (e.g. Notes Domino 4.61) have problems with
    # such an attribute-only request, they won't return any matches for a search.
    # If you experience these problems (no matching entry) set this to 0.
    search_attrsonly=1,
    search_tablistattrs=['cn','mail','homephone','telephonenumber','mobile'],
    # HTML template strings used to display entries in the table
    # of search results
    search_tdtemplate= {
#      'inetOrgPerson':r'%(cn)s &lt;%(mail)s&gt; Tel.: %(telephoneNumber)s or %(homePhoneNumber)s',
      'organization':r'&quot;%(o)s&quot; Tel.: %(telephoneNumber)s',
      'organizationalUnit':r'Org. Unit &quot;%(ou)s&quot; Tel.: %(telephoneNumber)s',
      'rfc822MailGroup':r'Mailing list %(cn)s &lt;%(mail)s&gt;, see %(labeledurl)s',
      'account':r'Account <strong>%(uid)s</strong>',
    },
    # Default for number of results shown per page
    search_resultsperpage=10,
    # HTML template file used for displaying entries of specific object class
    read_template={
      # 'object class':'pathname of HTML template file'
      'inetOrgPerson':os.path.join(web2ldapcnf.templates_dir,'read_inetOrgPerson.html'),
      'organizationalPerson':os.path.join(web2ldapcnf.templates_dir,'read_inetOrgPerson.html'),
      'organization':os.path.join(web2ldapcnf.templates_dir,'read_organization.html'),
      'posixAccount':os.path.join(web2ldapcnf.templates_dir,'read_posixAccount.html'),
    },
    # HTML template file used for displaying input forms for entries
    # of specific object class
    input_template={
      # 'object class':'pathname of HTML template file'
      'inetOrgPerson':os.path.join(web2ldapcnf.templates_dir,'inputform_inetOrgPerson.html'),
    },
    # HTML template file used for displaying operational attributes
    # in a footer when displaying a single entry
    read_operationalattrstemplate=os.path.join(web2ldapcnf.templates_dir,'read_operationalattrs.html'),
    # Number of columns for print output
    print_cols=4,
    # List of tuples (description,object class list) for displaying
    # buttons for quick-choosing object classes of new entries
    addform_oc_list=[
      ('Person','cn',['person','organizationalPerson','inetOrgPerson']),
      ('Company','o',['organization','labeledURIObject']),
      ('User','uid',['account','posixAccount','sambaAccount']),
      ('Org. Unit','ou',['organizationalUnit']),
      ('Group','cn',['groupOfNames']),
    ],
    # LDIF file used as locally stored pseudo LDAPv3 schema
    schema_uri='file://'+os.path.join(web2ldapcnf.etc_dir,'web2ldap/localschema.ldif'),
    groupadm_defs={
      'groupOfNames':       ('member',None),
      'groupOfUniqueNames': ('uniqueMember',None),
      'rfc822MailGroup':    ('mail','mail'),
      'mailGroup':          ('mgrprfc822mailmember','mail'),
      # Found on IBM SecureWay Directory
      'accessGroup':        ('member',None),
      # RFC2370
      'posixGroup':         ('memberUid','uid'),
    },
  ),

  'ldap:localhost': Web2LDAPConfig(
    description=u'My poorly configured LDAP host',
  ),

  'ldap://localhost:1389': Web2LDAPConfig(
    description=u'Local OpenLDAP 2.0.x test server',
    starttls=0,
    # Options for verifying SSL server certificate
    tls_cacertdir='',tls_cacertfile='/etc/httpd/ssl.crt/myCA-cacerts.pem',
    # Options for deploying client certificates for strong authentication
    tls_certfile='',tls_keyfile='',
  ),

  'ldaps://localhost:1636': Web2LDAPConfig(
    description=u'Local OpenLDAP 2.0.x test server access on separate SSL port',
    # StartTLS extended operation does not make sense here
    starttls=0,
    # Options for verifying SSL server certificate
    tls_cacertdir='',tls_cacertfile='/etc/httpd/ssl.crt/myCA-cacerts.pem',
    # Options for deploying client certificates for strong authentication
    tls_certfile='',tls_keyfile='',
  ),

  # This meant as an more complex example section
  # Adjust the settings to reflect your local LDAP installation
  'ldapi://%2Ftmp%2Fopenldap-REL_ENG_2_1/dc=stroeder,dc=de': Web2LDAPConfig(
    description=u'Private address book of Michael Strder',
    ssl_valid_dn=r'^/S=Stroeder/G=Michael/CN=Michael Stroeder/Email=michael@stroeder.com$',
    ssl_valid_idn=r'^/C=ZA/ST=Western Cape/L=Durbanville/O=Thawte/OU=Certificate Services/CN=Personal Freemail RSA 1999\.9\.16$',
    read_template={
      # 'object class':'pathname of HTML template file'
      'inetOrgPerson':os.path.join(web2ldapcnf.templates_dir,'read_inetOrgPerson.html'),
      'organization':os.path.join(web2ldapcnf.templates_dir,'read_organization.html'),
      'posixAccount':os.path.join(web2ldapcnf.templates_dir,'read_posixAccount.html'),
      'bankArrangement':os.path.join(web2ldapcnf.templates_dir,'read_bankArrangement.html'),
      'musician':os.path.join(web2ldapcnf.templates_dir,'read_musician.html'),
    },
    addform_oc_list=[
      ('Person','cn',['person','organizationalPerson','inetOrgPerson','msPerson']),
      ('Company','o',['organization','labeledURIObject']),
      ('User','uid',['account','posixAccount','sambaAccount']),
      ('Org. Unit','ou',['organizationalUnit']),
      ('Group','cn',['groupOfNames']),
    ],
    passwd_hashtypes=['ssha']
  ),

  # Special example for bibtex database of DFN:
  # - HTML templates for searchform, read and print are used.
  # - Search table contains most significant attributes.
  # (FYI: This site is not accessible anymore.)
  'ldap://os.directory.dfn.de/cn=bibtex,dc=directory,dc=dfn,dc=de': Web2LDAPConfig(
    description=u'Bibtex database of DFN',
    search_tablistattrs=['cn','bibtexTitle','bibtexAuthor','labeledURI'],
    searchform_template=os.path.join(
      web2ldapcnf.templates_dir,'searchform_bibtex.html'
    ),
    read_template={
      'bibtexEntry':os.path.join(
      web2ldapcnf.templates_dir,'read_bibtex.html'
    ),
    },
    print_cols=2,
    print_template={
      # 'object class':'pathname of printable HTML template file'
      'bibtexEntry':os.path.join(
        web2ldapcnf.templates_dir,'print_bibtex.html'
      ),
    },
  ),

  # Special example for IBM Directory Server 5.1
  # - HTML templates for searchform, read and print are used.
  # - Search table contains most significant attributes.
  # (FYI: This site is not accessible anymore.)
  'ldap://localhost:7389': Web2LDAPConfig(
    requested_attrs=[
      'ibm-audit','ibm-auditAdd','ibm-auditBind','ibm-auditDelete','ibm-auditExtOpEvent','ibm-auditFailedOpOnly','ibm-auditLog','ibm-auditModify','ibm-auditModifyDN','ibm-auditSearch','ibm-auditUnbind'
    ],
  ),

}
