"""
w2lapp.cnf.py: read configuration data

web2ldap - a web-based LDAP Client,
see http://www.web2ldap.de for details

(c) by Michael Stroeder <michael@stroeder.com>

This module is distributed under the terms of the
GPL (GNU GENERAL PUBLIC LICENSE) Version 2
(see http://www.gnu.org/copyleft/gpl.html)

$Id: cnf.py,v 1.18 2002/11/15 07:38:07 michael Exp $
"""

import signal,w2lapp.core,ldapurl,ldapsession

import web2ldapcnf,web2ldapcnf.hosts,web2ldapcnf.misc,web2ldapcnf.standalone,web2ldapcnf.fastcgi
from web2ldapcnf import hosts,misc,standalone,fastcgi
from web2ldapcnf.hosts import ldap_def
from ldap.cidict import cidict

ldap_def_cidict = cidict(ldap_def)

def HUPSignalHandler(signum,frame):
  assert signum==signal.SIGHUP, \
    'Received wrong signal: Expected signal.SIGHUP, got %d' % (signum)
  reload(web2ldapcnf)
  reload(web2ldapcnf.hosts)
  reload(web2ldapcnf.misc)
  reload(web2ldapcnf.standalone)
  reload(web2ldapcnf.fastcgi)
  from web2ldapcnf import hosts,misc,standalone,fastcgi
  from web2ldapcnf.hosts import ldap_def
  ldap_def_cidict = cidict(ldap_def)
  w2lapp.core.session.expireDeactivate = w2lapp.cnf.misc.session_relogin
  w2lapp.core.session.expireRemove = w2lapp.cnf.misc.session_remove
  ldap._trace_level = web2ldapcnf.misc.trace_level
  w2lapp.schema.parse_fake_schema()

try:
  # Set signal handler for SIGHUP
  signal.signal(signal.SIGHUP,HUPSignalHandler)
except AttributeError:
  pass

def GetParam(ls,k,default=''):
  """
  Get a parameter determined by string-key k
  depending on current ls
  """
  if isinstance(ls,ldapurl.LDAPUrl):
    ldap_url = ('%s://%s/%s' % (ls.urlscheme,ls.hostport,ls.dn)).lower()
    conn_uri = ('%s://%s' % (ls.urlscheme,ls.hostport)).lower()
  elif isinstance(ls,ldapsession.LDAPSession):
    ldap_url = '/'.join((str(ls.uri),str(ls.getSearchRoot(ls._dn)))).lower()
    conn_uri = ls.uri
  else:
    raise TypeError,'ls has to be instance of ldapurl.LDAPUrl or ldapsession.LDAPSession'

  if ldap_def_cidict.has_key(ldap_url) and ldap_def_cidict[ldap_url].__dict__.has_key(k):
    # Parameter found for LDAP URL ldap://host/current dn
    result = ldap_def_cidict[ldap_url].__dict__[k]
  elif ldap_def_cidict.has_key(conn_uri) and ldap_def_cidict[conn_uri].__dict__.has_key(k):
    # Parameter found for hostname
    result = ldap_def_cidict[conn_uri].__dict__[k]
  elif ldap_def_cidict.has_key('_') and ldap_def_cidict['_'].__dict__.has_key(k):
    # found parameter in default section
    result = ldap_def_cidict['_'].__dict__[k]
  else:
    # no parameter found in configuration module
    result = default

  return result
