"""
w2lapp.groupadm.py: add/delete user entry to/from group entries

web2ldap - a web-based LDAP Client,
see http://www.web2ldap.de for details

(c) by Michael Stroeder <michael@stroeder.com>

This module is distributed under the terms of the
GPL (GNU GENERAL PUBLIC LICENSE) Version 2
(see http://www.gnu.org/copyleft/gpl.html)

$Id: groupadm.py,v 1.45 2002/10/03 22:00:44 michael Exp $
"""

import ldap,ldap.cidict,ldaputil.base, \
       pyweblib.forms,w2lapp.core,w2lapp.gui


ACTION2MODTYPE = {'add':ldap.MOD_ADD,'remove':ldap.MOD_DELETE}

REQUESTED_GROUP_ATTRS = ['objectClass','cn','description']

def w2l_GroupAdm(sid,outf,command,form,ls,dn):

  groupadm_defs = ldap.cidict.cidict(w2lapp.cnf.GetParam(ls,'groupadm_defs',{}))
  if not groupadm_defs:
    raise w2lapp.core.ErrorExit('<em>groupadm_defs</em> empty or not set.')
  groupadm_defs_keys = groupadm_defs.keys()

  sub_schema = ls.retrieveSubSchema(dn,w2lapp.cnf.GetParam(ls,'_schema',None))

  result_dnlist = ls.readEntry(dn,['objectClass'])
  if not result_dnlist:
    raise w2lapp.core.ErrorExit("""
      No search result when reading entry <strong>%s</strong>.""" % (
        w2lapp.gui.DisplayDN(sid,form,ls,dn)
      )
    )

  user_entry = ldap.schema.Entry(sub_schema,dn,result_dnlist[0][1])

  group_search_root = form.getInputValue('group_search_root',[ls.getSearchRoot(dn)])[0]

  filter_components = []
  for oc in groupadm_defs.keys():
    group_member_attrtype,user_entry_attrtype = groupadm_defs[oc]
    if user_entry_attrtype is None:
      user_entry_attrvalue = dn.encode(ls.charset)
    else:
      try:
        user_entry_attrvalue = user_entry[user_entry_attrtype][0]
      except KeyError:
        continue
    filter_components.append(
      (
        oc,
        group_member_attrtype,
        ldaputil.base.escape_filter_chars(user_entry_attrvalue)
      )
    )

  #################################################################
  # Search all the group entries
  #################################################################

  all_group_filterstr = '(|%s)' % (''.join(
    [
      '(objectClass=%s)' % (oc)
      for oc,attr_type,attr_value in filter_components
    ]
  ))
    
  all_groups_dict = {}

  for group_dn,group_entry in ls.l.search_st(
    group_search_root.encode(ls.charset),
    ldap.SCOPE_SUBTREE,
    all_group_filterstr,
    attrlist=REQUESTED_GROUP_ATTRS,attrsonly=0,timeout=ls.timeout
  ):
    if group_dn!=None:
      all_groups_dict[group_dn] = ldap.cidict.cidict(group_entry)

  all_group_entries = all_groups_dict.keys()
  all_group_entries.sort()

  #################################################################
  # Apply changes to group membership
  #################################################################

  if 'groupadm_add' in form.inputFieldNames or \
     'groupadm_remove' in form.inputFieldNames:

    for action in ['add','remove']:
      for action_group_dn in form.getInputValue('groupadm_%s'%action,[]):
        group_dn = action_group_dn.encode(ls.charset)
        if not all_groups_dict.has_key(group_dn):
          continue
        modlist = []
        for oc in groupadm_defs_keys:
          if oc.lower() in [ v.lower() for v in all_groups_dict[group_dn]['objectClass'] ]:
            group_member_attrtype,user_entry_attrtype = groupadm_defs[oc]
            if user_entry_attrtype is None:
              member_value = dn.encode(ls.charset)
            else:
              if not user_entry.has_key(user_entry_attrtype):
                raise w2lapp.core.ErrorExit("""
                  Object class %s requires entry to have member attribute %s.""" % (
                    objectClass,user_entry_attrtype
                  )
                )                
              member_value = user_entry[user_entry_attrtype][0]
            modlist.append((ACTION2MODTYPE[action],group_member_attrtype,member_value))
        ls.modifyEntry(group_dn,modlist)

  #################################################################
  # Search for groups the entry is member of
  #################################################################

  remove_group_filterstr = '(|%s)' % (''.join(
    [
      '(&(objectClass=%s)(%s=%s))' % (oc,attr_type,attr_value)
      for oc,attr_type,attr_value in filter_components
    ]
  ))

  remove_groups_dict = {}
  
  for group_dn,group_entry in ls.l.search_st(
    group_search_root.encode(ls.charset),
    ldap.SCOPE_SUBTREE,
    remove_group_filterstr,
    attrlist=[],attrsonly=0,timeout=ls.timeout
  ):
    if group_dn!=None:
      remove_groups_dict[group_dn] = None

  remove_group_dns = remove_groups_dict.keys()
  remove_group_dns.sort()

  remove_groups = [
    (
      group_dn,
      w2lapp.core.utf2display(
        form.accept_charset,all_groups_dict[group_dn].get(
          'cn',[ldaputil.base.SplitRDN(group_dn)[0]]
        )[0],escape_entities=0
      )
    )
    for group_dn in remove_group_dns
  ]

  #########################################################
  # Sort out groups the entry is not(!) a member of
  #########################################################

  add_groups = [
    (
      group_dn,
      w2lapp.core.utf2display(
        form.accept_charset,all_groups_dict[group_dn].get(
          'cn',[ldaputil.base.SplitRDN(group_dn)[0]]
        )[0],escape_entities=0
      )
    )
    for group_dn in all_group_entries
    if not remove_groups_dict.has_key(group_dn)
  ]

  #########################################################
  # HTML output
  #########################################################

  w2lapp.gui.TopSection(
    sid,outf,form,ls,dn,
    'Group selection table',
    w2lapp.gui.MainMenu(sid,form,ls,dn),
    context_menu_list=[]
  )

  add_select_field = pyweblib.forms.Select(
    'groupadm_add','Add to group',1000,
    options=add_groups,default=[],size=15,multiSelect=1
  )
  remove_select_field = pyweblib.forms.Select(
    'groupadm_remove','remove to group',1000,
    options=remove_groups,default=[],size=15,multiSelect=1
  )

  group_search_root_field = w2lapp.gui.SearchRootField(
    form,ls,dn,name='group_search_root'
  )
  group_search_root_field.setDefault(group_search_root)

  outf.write("""<div id="MessageDiv">
<form action="%s/groupadm/%s"
      method="POST"
      enctype="application/x-www-form-urlencoded"
      accept-charset="%s"
>
  <input type="hidden" name="dn" value="%s">
  <p>Group entries are searched below %s.</p>
  <input type="submit" value="Change group membership">
  <table summary="Group select fields">
""" % (
    form.script_name,sid,form.accept_charset,
    w2lapp.core.utf2display(form.accept_charset,dn),
    group_search_root_field.inputHTML()
  ))

  outf.write("""
    <tr>
      <td width="50%%">Add to...</td>
      <td width="50%%">Remove from...</td>
    </tr>
    <tr>
      <td width="50%%">%s</td>
      <td width="50%%">%s</td>
    </tr>
""" % (add_select_field.inputHTML(),remove_select_field.inputHTML()))

  outf.write("""  
  </table>
</form>
""")

  outf.write('<dl>\n')
  # Output a legend of all group entries
  for group_dn in all_group_entries:
    group_entry = all_groups_dict[group_dn]
    outf.write('<dt>%s | %s<dt>\n<dd>%s<br>\n(%s)<br>\n%s<dd>\n' % (
      ', '.join(group_entry.get('cn',[])),
      w2lapp.gui.W2L_Anchor(
	form,'read','Read',sid,[('dn',group_dn)]
      ),
      w2lapp.core.utf2display(form.accept_charset,group_dn),
      ', '.join(group_entry.get('objectClass',[])),
      '<br>'.join(group_entry.get('description',[]))
    ))
  outf.write('</dl>\n')

  outf.write('</div>\n')

  w2lapp.gui.PrintFooter(outf,form)
