"""
w2lapp.modify.py: modify an entry

web2ldap - a web-based LDAP Client,
see http://www.web2ldap.de for details

(c) by Michael Stroeder <michael@stroeder.com>

This module is distributed under the terms of the
GPL (GNU GENERAL PUBLIC LICENSE) Version 2
(see http://www.gnu.org/copyleft/gpl.html)

$Id: modify.py,v 1.54 2002/11/20 13:42:18 michael Exp $
"""

import ldap,ldif,ldap.schema,ldap.modlist,msbase,ldaputil.base, \
       pyweblib.forms, \
       w2lapp.core,w2lapp.cnf,w2lapp.gui,w2lapp.addmodifyform


def GetEntryfromInputForm(form,ls,dn,schema):

  e = ldap.schema.Entry(schema,dn.encode(ls.charset),{})

  try:
    in_ldif = form.field['in_ldif'].getLDIFRecords()
  except ValueError,e:
    raise w2lapp.core.ErrorExit('LDIF parsing error: '+str(e))
  else:
    if in_ldif:
      e.update(in_ldif[0][1])

  in_attrtype_list = [
    a.encode('ascii')
    for a in form.getInputValue('in_attrtype',[])
  ]
  in_value_list = [
    a.encode(ls.charset)
    for a in form.getInputValue('in_value',[])
  ]
  if len(in_attrtype_list)!=len(in_value_list):
    raise w2lapp.core.ErrorExit('Different count of attribute types and values.')
  for i in range(len(in_attrtype_list)):
    attr_type = in_attrtype_list[i]
    attr_value = in_value_list[i]
    if attr_type:
      if not e.has_key(attr_type):
        e[attr_type] = []
      if attr_value:
	e[attr_type].append(attr_value)
  return e # GetEntryfromInputForm()


def ValidateEntryInput(e):
  errornous_attrs = []
  return errornous_attrs
  for attr_type in e.keys():
    syntax_class = w2lapp.schema.syntaxes.syntax_registry.get(
      attr_type,w2lapp.schema.syntaxes.LDAPSyntax
    )
    for attr_value in e[attr_type]:
      if attr_value:
        attribute = syntax_class(attr_type,attr_value)
        if not attribute.validate():
          errornous_attrs.append(attr_type)
          break
  return errornous_attrs


def ModlistTable(sid,form,ls,dn,schema,modlist):
  """
  Return a string containing a HTML table showing modifiy operations
  and attr type/value pairs
  """
  s = []
  s.append('<table summary="Modify list">')
  for mod_op,mod_type,mod_val in modlist:
    if mod_val is None:
      mod_val_str = ''
    elif w2lapp.schema.no_humanreadable_attr(schema,mod_type):
      mod_val_str = '%s bytes of binary data' % (
        ' + '.join(
          [ str(len(x)) for x in mod_val ]
        )
      )
    else:
      mod_val_str = '<br>'.join([
        pyweblib.forms.escapeHTML(repr(v))
        for v in mod_val
      ])
    s.append('<tr><td>%s</td><td>%s</td><td>%s</td></tr>' % (
        ldif.MOD_OP_STR[mod_op],mod_type,mod_val_str
      )
    )
  s.append('</table>')
  return '\n'.join(s) # ModlistTable()


##############################################################################
# Modify existing entry
##############################################################################

def w2l_Modify(sid,outf,command,form,ls,dn):

  sub_schema = ls.retrieveSubSchema(dn,w2lapp.cnf.GetParam(ls,'_schema',None))

  new_entry = GetEntryfromInputForm(form,ls,dn,sub_schema)

  errornous_attrs = ValidateEntryInput(new_entry)
  if errornous_attrs:
    w2lapp.addmodifyform.w2l_AddModifyForm(
      sid,outf,command+'form',form,ls,dn,
      Msg='Syntax check failed for the following attributes: %s' % (
        ', '.join(errornous_attrs)
      ),
      entry=new_entry,
      skip_oc_input=1
    )
    return
    
  existing_attrs = new_entry.keys()

  # Read modifiable attributes of entry
  result_dnlist = ls.readEntry(dn,existing_attrs,no_cache=1)
  if not result_dnlist:
    raise w2lapp.core.ErrorExit("""
      No search result when reading entry <strong>%s</strong>.""" % (
        w2lapp.gui.DisplayDN(sid,form,ls,dn)
      )
    )

  old_entry = w2lapp.addmodifyform.read_old_entry(ls,dn,sub_schema)
  old_objectclasses = old_entry.get('objectClass',[])

  modlist = ldap.modlist.modifyModlist(
    old_entry,new_entry,
    ignore_oldexistent=1,
  )

  if modlist:
    # Send modify-list to host
    try:
      ls.modifyEntry(dn,modlist)
    except ( ldap.OBJECT_CLASS_VIOLATION,
             ldap.TYPE_OR_VALUE_EXISTS,
             ldap.UNDEFINED_TYPE,
             ldap.INVALID_SYNTAX,
             ldap.CONSTRAINT_VIOLATION ),e:
      w2lapp.addmodifyform.w2l_AddModifyForm(
        sid,outf,command+'form',form,ls,dn,
        Msg=w2lapp.gui.LDAPError2ErrMsg(e),
        entry=new_entry,
        skip_oc_input=1
      )
      return
    else:
      # delete all cache entries referencing to old DN
      UserMsg = 'Modified entry %s with your input data:</p>\n%s' % (
        w2lapp.gui.DisplayDN(sid,form,ls,dn),
        ModlistTable(sid,form,ls,dn,sub_schema,modlist).encode(form.accept_charset)
      )
  else:
    UserMsg = 'No attributes modified of entry %s.' % (w2lapp.gui.DisplayDN(sid,form,ls,dn))

  # Output comes here
  w2lapp.gui.SimpleMessage(
    sid,outf,form,ls,dn,
    'Modify result',
    UserMsg,
    main_menu_list=w2lapp.gui.MainMenu(sid,form,ls,dn),
    context_menu_list=w2lapp.gui.ContextMenuSingleEntry(sid,form,ls,dn)
  )

