"""
w2lapp.schema.syntaxes: classes for known attribute types

web2ldap - a web-based LDAP Client,
see http://www.web2ldap.de for details

(c) by Michael Stroeder <michael@stroeder.com>

This module is distributed under the terms of the
GPL (GNU GENERAL PUBLIC LICENSE) Version 2
(see http://www.gnu.org/copyleft/gpl.html)

$Id: syntaxes.py,v 1.27 2002/12/02 10:18:38 michael Exp $
"""

import re,imghdr,urllib,utctime,ldap,ldapurl,pyweblib.forms, \
       w2lapp.viewer,w2lapp.gui,mspki.util

from w2lapp.core import utf2display

from ldaputil.base import is_dn

try:
  from ldapoidreg import oid as oid_desc_reg
except ImportError:
  oid_desc_reg = {}


class SyntaxRegistry(ldap.cidict.cidict):

  def register(self,syntax_oid,attrTypes):
    for a in attrTypes:
      self[a.strip()] = syntax_oid

  def syntaxClass(self,schema,attrtype_nameoroid):
    attrtype_oid = schema.getoid(ldap.schema.AttributeType,attrtype_nameoroid)
    if self.has_key(attrtype_oid):
      syntax_class = oid2syntax[self[attrtype_oid]]
    else:
      attrtype_se = schema.get(attrtype_oid,None)
      if attrtype_se is None:
        syntax_class = LDAPSyntax
      else:
        syntax_class = oid2syntax.get(attrtype_se.syntax,LDAPSyntax)
    return syntax_class 


host_pattern = '([0-9]+\.[0-9]+\.[0-9]+\.[0-9]+|[a-zA-Z]+[a-zA-Z0-9-]*(\.[a-zA-Z]+[a-zA-Z0-9-]*)*)?(:[0-9]*|)?'
url_pattern  = r'^(ftp|http|https|news|snews|ldap|ldaps|mailto):(|//)[^ ]*'
url_regex  = re.compile(url_pattern)
labeleduri_regex = re.compile(url_pattern+r' .*')
timestamp_pattern = r'^([0-9]){12,14}(.[0-9]+)*Z$'
timestamp_regex  = re.compile(timestamp_pattern)
mail_pattern = r'^[\w@.=/_ -]+@[a-zA-Z0-9-]+(\.[a-zA-Z0-9-]+)*$'
mail_regex = re.compile(mail_pattern)
# See http://developer.novell.com/ndk/doc/ndslib/nds__enu/data/hniuyp90.html#hniuyp90
novell_network_address_types = (
  ('NT_IPX','Internet Packet Exchange (IPX) network address'),
  ('NT_IP','Internet Protocol (IP) network address'),
  ('NT_SDLC','Synchronous Data Link Control (SDLC) address'),
  ('NT_TOKENRING_ETHERNET','Token ring on Ethernet MAC address'),
  ('NT_OSI','Open Systems Interconnection (OSI) address'),
  ('NT_APPLETALK','AppleTalk network address'),
  ('NT_NETBEUI','NetBIOS Extended User Interface (NetBEUI) address'),
  ('NT_SOCKADDR','Socket address'),
  ('NT_UDP','User Datagram Protocol (UDP) address'),
  ('NT_TCP','Transmission Control Protocol (TCP) address'),
  ('NT_UDP6','User Datagram Protocol (UDP), version 6, address'),
  ('NT_TCP6','Transmission Control Protocol (TCP), version 6, address'),
  ('NT_INTERNAL','Reserved'),
  ('NT_URL','Uniform Resource Locator address (added for NDS 8)'),
  ('NT_COUNT','Returns the maximum number of network address types defined.'),
)


####################################################################
# Classes of known syntaxes
####################################################################

class LDAPSyntax:
  oid=''
  desc='Generic LDAP syntax'
  maxLen = w2lapp.cnf.misc.input_maxfieldlen
  maxValues = w2lapp.cnf.misc.input_maxattrs
  reObj=re.compile('^.*$')

  def __init__(self,attrType,attrValue=None,charset='utf-8'):
    self._charset = charset
    self.attrType = attrType
    self.attrValue = attrValue
#    if attrValue:
#      self.setValue(attrValue)

  def validate(self):
    return self.reObj.match(self.attrValue)!=None

  def setValue(self,attrValue):
    self.attrValue = attrValue

  def getValue(self):
    return self.attrValue

  def formField(self):
    return pyweblib.forms.Input(
      self.attrType,
      ': '.join([self.attrType,self.desc]),
      self.maxLen,self.maxValues,self.reObj.pattern,default=self.attrValue,size=50
    )

  def displayValue(self,sid,form,ls,dn,valueindex=0,commandbutton=0):
    attrtype_lower = self.attrType.lower()
    if attrtype_lower=='pgpkey':
      return '<pre>%s</pre>' % (self.attrValue)
    elif ls.__dict__.get('vendorName','').startswith('Novell') and attrtype_lower=='networkaddress':
      address_type,address_data = self.attrValue.split('#',1)
      address_type_ident,address_type_desc = novell_network_address_types[int(address_type)]
      return '<dl><dt>%s (%s)</dt><dd>%s</dd></dl>' % (
        address_type_ident,
        pyweblib.forms.escapeHTML(address_type_desc),
        pyweblib.forms.escapeHTML(repr(address_data))
      )
    elif ldapurl.isLDAPUrl(self.attrValue):
      if commandbutton:
        commandbuttonstr = w2lapp.gui.LDAPURLButton(sid,form,ls,self.attrValue)
      else:
        commandbuttonstr = ''
      return '<table><tr><td>%s</td><td><a href="%s" target="_ldapurl">%s</a></td></tr></table>' % (
               commandbuttonstr,
	       self.attrValue.encode(form.accept_charset),
	       utf2display(form.accept_charset,self.attrValue)
	     )
    # Check for RFC822 mail address
    rm = mail_regex.match(self.attrValue)
    if rm!=None:
      return w2lapp.gui.MailToURL(self.attrValue)
    # Check for DN
    if is_dn(self.attrValue):
      return w2lapp.gui.DisplayDN(sid,form,ls,self.attrValue,commandbutton)
    # Check for URL
    rm = url_regex.search(self.attrValue)
    if rm!=None:
      rm = labeleduri_regex.match(self.attrValue)
      if rm!=None:
        url,label = self.attrValue.split(' ',1)
        return '<a href="%s/urlredirect?%s" target="_labeledurl">%s</a>' % (form.script_name,url,utf2display(form.accept_charset,label))
      else:
        return '<a href="%s/urlredirect?%s" target="_labeledurl">%s</a>' % (form.script_name,self.attrValue,self.attrValue)
    # Check for timestamp
    rm = timestamp_regex.match(self.attrValue)
    if rm!=None:
      return utctime.strftimeiso8601(utctime.strptime(self.attrValue))
    # No special format detected
    return utf2display(form.accept_charset,self.attrValue)


class Binary(LDAPSyntax):
  oid = '1.3.6.1.4.1.1466.115.121.1.5'
  desc = 'Binary'

  def validate(self):
    return 1

  def displayValue(self,sid,form,ls,dn,valueindex=0,commandbutton=0):
    return w2lapp.gui.W2L_Anchor(
      form,'read','View/Load',sid,
      [('dn',dn),('read_attr',self.attrType),('read_attrindex',str(valueindex))]
    )


class Audio(Binary):
  oid = '1.3.6.1.4.1.1466.115.121.1.4'
  desc = 'Audio'

  def validate(self):
    return w2lapp.viewer.what_sndhdr(self.attrValue)=='au'

  def displayValue(self,sid,form,ls,dn,valueindex=0,commandbutton=0):
    mimetype = w2lapp.viewer.snd_mimetype(self.attrValue)
    return """
      <embed
        type="%s"
        autostart="false"
        src="%s/read/%s?dn=%s&read_attr=%s&read_attrindex=%d"
      >
      %d bytes of audio data (%s)
      """ % (
        mimetype,
        form.script_name,sid,
        urllib.quote_plus(utf2display(form.accept_charset,dn,escape_entities=0)),
        urllib.quote_plus(self.attrType),
        valueindex,
        len(self.attrValue),
        mimetype
      )


class Certificate(Binary):
  oid = '1.3.6.1.4.1.1466.115.121.1.8'
  desc = 'Certificate Revocation List'

  def displayValue(self,sid,form,ls,dn,valueindex=0,commandbutton=0):
    return w2lapp.gui.W2L_Anchor(
      form,'read','View/Load',sid,
      [
        ('dn',dn),
        ('read_attr',self.attrType),
        ('read_attrindex',str(valueindex)),
        ('read_attrmode','view'),
      ]
    )


class CertificateRevocationList(Certificate):
  oid = '1.3.6.1.4.1.1466.115.121.1.9'
  desc = 'Certificate Revocation List'


class DirectoryString(LDAPSyntax):
  oid = '1.3.6.1.4.1.1466.115.121.1.15'
  desc = 'Directory String'

  def setValue(self,attrValue):
    self.attrValue = unicode(attrValue,self._charset)

  def getValue(self):
    return self.attrValue.encode(self._charset)

  def displayValue(self,sid,form,ls,dn,valueindex=0,commandbutton=0):
    return utf2display(form.accept_charset,self.attrValue)


class DistinguishedName(DirectoryString):
  oid = '1.3.6.1.4.1.1466.115.121.1.12'
  desc = 'Distinguished Name'

  def validate(self):
    return is_dn(self.attrValue)

  def displayValue(self,sid,form,ls,dn,valueindex=0,commandbutton=0):
    return w2lapp.gui.DisplayDN(sid,form,ls,self.attrValue,commandbutton=commandbutton)

  def formField(self):
    return w2lapp.gui.DistinguishedNameInput(
      self.attrType,': '.join([self.attrType,self.desc]),default=self.attrValue
    )


class NameAndOptionalUID(DistinguishedName):
  oid = '1.3.6.1.4.1.1466.115.121.1.34'
  desc = 'Name And Optional UID'

  def validate(self):
    value = '#'.split(self.attrValue)
    return is_dn(value[0])

  def displayValue(self,sid,form,ls,dn,valueindex=0,commandbutton=0):
    value = self.attrValue.split('#')
    dn_str = w2lapp.gui.DisplayDN(
      sid,form,ls,value[0],commandbutton=commandbutton
    )
    if len(value)==1 or not value[1]:
      return dn_str
    else:
      return ' | '.join([value[1],dn_str])
      

class Boolean(LDAPSyntax):
  oid = '1.3.6.1.4.1.1466.115.121.1.7'
  desc = 'Boolean'

  def validate(self):
    return (self.attrValue=='TRUE') or (self.attrValue=='FALSE')

  def setValue(self,attrValue):
    self.attrValue = {0:'FALSE',1:'TRUE'}[attrValue.__nonzero__()]

  def getValue(self):
    return self.attrValue

  def displayValue(self,sid,form,ls,dn,valueindex=0,commandbutton=0):
    return self.attrValue.encode('ascii')


class CaseExactString(DirectoryString):
  oid = '1.3.6.1.4.1.1466.115.121.1.26'
  desc = 'Case Exact String'


class GeneralizedTime(LDAPSyntax):
  oid = '1.3.6.1.4.1.1466.115.121.1.24'
  desc = 'Generalized Time'
  reObj=timestamp_regex

  def displayValue(self,sid,form,ls,dn,valueindex=0,commandbutton=0):
    try:
      return utctime.strftimeiso8601(utctime.strptime(self.attrValue)).encode('ascii')
    except ValueError:
      return utf2display(form.accept_charset,self.attrValue)


class UTCTime(GeneralizedTime):
  oid = '1.3.6.1.4.1.1466.115.121.1.53'
  desc = 'UTC Time'


class IA5String(LDAPSyntax):
  oid = '1.3.6.1.4.1.1466.115.121.1.26'
  desc = 'IA5 String'

  def validate(self):
    try:
      self.attrValue.encode('ascii')
    except UnicodeError:
      return 0
    else:
      return LDAPSyntax.validate(self)

  def displayValue(self,sid,form,ls,dn,valueindex=0,commandbutton=0):
    return self.attrValue.encode('ascii')


class Integer(LDAPSyntax):
  oid = '1.3.6.1.4.1.1466.115.121.1.27'
  desc = 'Integer'
  reObj= re.compile('^[0-9]+$')

  def displayValue(self,sid,form,ls,dn,valueindex=0,commandbutton=0):
    return self.attrValue.encode('ascii')


class RFC822Address(IA5String):
  oid = 'RFC822Address-oid'
  desc = 'RFC 822 mail address'
  reObj= re.compile(mail_pattern)

  def displayValue(self,sid,form,ls,dn,valueindex=0,commandbutton=0):
    return w2lapp.gui.MailToURL(self.attrValue)


class Uri(DirectoryString):
  oid = 'Uri-OID'
  desc = 'URI'
  reObj = url_regex

  def displayValue(self,sid,form,ls,dn,valueindex=0,commandbutton=0):
    try:
      url,label = self.attrValue.split(' ',1)
    except ValueError:
      return '<a href="%s/urlredirect?%s" target="_labeledurl">%s</a>' % (form.script_name,self.attrValue,self.attrValue)
    else:
      return '<a href="%s/urlredirect?%s" target="_labeledurl">%s</a>' % (form.script_name,url,utf2display(form.accept_charset,label))


class JPEGImage(Binary):
  oid = '1.3.6.1.4.1.1466.115.121.1.28'
  desc = 'JPEG image'

  def validate(self):
    return imghdr.what(None,self.attrValue)=='jpeg'

  def displayValue(self,sid,form,ls,dn,valueindex=0,commandbutton=0):
    return """
      <img
        src="%s/read/%s?dn=%s&read_attr=%s&amp;read_attrindex=%d&amp;read_attrmode=load"
      >
      <br>
      %d bytes of image data
      """ % (
      form.script_name,sid,
      urllib.quote_plus(utf2display(form.accept_charset,dn,escape_entities=0)),
      urllib.quote_plus(self.attrType),
      valueindex,len(self.attrValue),
    )


class OID(LDAPSyntax):
  oid = '1.3.6.1.4.1.1466.115.121.1.38'
  desc = 'OID'
  regex = '([0-9]+.)*[0-9]'

  def displayValue(self,sid,form,ls,dn,valueindex=0,commandbutton=0):
    attrValue = self.attrValue.encode('ascii')
    if oid_desc_reg.has_key(attrValue):
      name,description,reference = oid_desc_reg[self.attrValue]
      return '<strong>%(name)s</strong> (%(attrValue)s):<br>%(description)s (see %(reference)s)' % vars()
    else:
      return attrValue


class LDAPUrl(DirectoryString):
  oid = 'LDAPUrl-oid'
  desc = 'LDAP URL'

  def displayValue(self,sid,form,ls,dn,valueindex=0,commandbutton=0):
    try:
      if commandbutton:
        commandbuttonstr = w2lapp.gui.LDAPURLButton(sid,form,ls,self.attrValue)
      else:
        commandbuttonstr = ''
      return '<table><tr><td>%s</td><td><a href="%s" target="_ldapurl">%s</a></td></tr></table>' % (
               commandbuttonstr,
	       self.attrValue.encode(form.accept_charset),
	       utf2display(form.accept_charset,self.attrValue)
	     )
    except ValueError,e:
      return '<strong>Not a valid LDAP URL:</strong> %s' % utf2display(form.accept_charset,repr(self.attrValue))


class OctetString(LDAPSyntax):
  oid = '1.3.6.1.4.1.1466.115.121.1.40'
  desc = 'Octet String'

  def validate(self):
    return 1

  def displayValue(self,sid,form,ls,dn,valueindex=0,commandbutton=0):
    try:
      return self.attrValue.encode(form.accept_charset)
    except UnicodeError:
      return '<code><font size="-1">%s</font></code>' % mspki.util.HexString(
        self.attrValue,
        delimiter=':',wrap=64,linesep='<br>'
      )
    

class PostalAddress(DirectoryString):
  oid = '1.3.6.1.4.1.1466.115.121.1.41'
  desc = 'Postal Address'
  reObj=re.compile('^.*$',re.S+re.M)
  def _split_lines(self,v):
    return v.split('$')

  def displayValue(self,sid,form,ls,dn,valueindex=0,commandbutton=0):
    lines = [
      utf2display(form.accept_charset,l)
      for l in self._split_lines(self.attrValue)
    ]
    return '<br>'.join(lines)

  """
  def validate(self):
    return self.reObj.match(self.attrValue)!=None

  def setValue(self,attrValue):
    attrValue = unicode(attrValue,self._charset)
    lines = attrValue.split('\n')
    for i in range(len(lines)):
      if lines[i].endswith('\r'):
        lines[i] = lines[i][:-1]
    self.attrValue = '$'.join(lines)

  def formField(self):
    return pyweblib.forms.Textarea(
      self.attrType,
      ': '.join([self.attrType,self.desc]),
      self.maxLen,self.maxValues,
      self.reObj.pattern,
      default=u'\r\n'.join(self._split_lines(self.attrValue)),
      rows=3,cols=40
    )
  """


class PrintableString(LDAPSyntax):
  oid = '1.3.6.1.4.1.1466.115.121.1.44'
  desc = 'Printable String'
  reObj= re.compile('^[a-zA-Z0-9"()+,-./:?  -]*$')

  def displayValue(self,sid,form,ls,dn,valueindex=0,commandbutton=0):
    return self.attrValue.encode('ascii')


class TelephoneNumber(LDAPSyntax):
  oid = '1.3.6.1.4.1.1466.115.121.1.50'
  desc = 'Telephone Number'
  reObj= re.compile('^\+[0-9 ]+$')

  def displayValue(self,sid,form,ls,dn,valueindex=0,commandbutton=0):
    return self.attrValue.encode('ascii')


class FacsimileTelephoneNumber(TelephoneNumber):
  oid = '1.3.6.1.4.1.1466.115.121.1.22'


class ObjectGUID(LDAPSyntax):
  oid = 'ObjectGUID-oid'
  desc = 'Object GUID'

  def displayValue(self,sid,form,ls,dn,valueindex=0,commandbutton=0):
    objectguid_str = ''.join(['%02X' % ord(c) for c in self.attrValue])
    return ldapurl.LDAPUrl(
      ldapUrl=ls.uri,
      dn='GUID=%s' % (objectguid_str),
      who=None,cred=None
    ).htmlHREF(
      hrefText=objectguid_str,
      hrefTarget='_ldapurl'
    )


syntax_registry = SyntaxRegistry()

oid2syntax = {}

for _name in dir():
  o = eval(_name)
  if hasattr(o,'oid'):
    oid2syntax[o.oid] = o

syntax_registry.register(
  OID.oid,
  [
    '1.2.826.0.1050.11.0','ogSupportedProfile',
    '1.3.6.1.4.1.1466.101.120.13','supportedControl',
    '1.3.6.1.4.1.1466.101.120.7','supportedExtension',
    '1.3.6.1.4.1.4203.1.3.5','supportedFeatures',
  ]
)

syntax_registry.register(
  RFC822Address.oid,
  [
    '0.9.2342.19200300.100.1.3',    # mail, rfc822Mailbox
    '2.16.840.1.113730.3.1.13',     # mailLocalAddress
    '2.16.840.1.113730.3.1.17',     # mailForwardingAddress
    '2.16.840.1.113730.3.1.30',     # mgrpRFC822MailMember
    '2.16.840.1.113730.3.1.47',     # mailRoutingAddress
    '1.2.840.113549.1.9.1',         # email, emailAddress, pkcs9email
    '2.16.840.1.113678.2.2.2.2.15', # Domino/LDAP: MailAddress
  ]
)

syntax_registry.register(
  JPEGImage.oid,
  [
    '0.9.2342.19200300.100.1.60', # jpegPhoto
  ]
)

syntax_registry.register(
  Audio.oid,
  [
    '0.9.2342.19200300.100.1.55', # audio
  ]
)

syntax_registry.register(
  Uri.oid,[
    '1.3.6.1.4.1.250.1.57', # labeledURI
  ]
)

syntax_registry.register(
  Boolean.oid,[
    '2.5.18.9', # hasSubordinates
  ]
)

syntax_registry.register(
  PostalAddress.oid,[
    '2.5.4.16',                   # postalAddress
    '2.5.4.26',                   # registeredAddress
    '0.9.2342.19200300.100.1.39', # homePostalAddress
  ]
)

syntax_registry.register(
  LDAPUrl.oid,[
    '2.16.840.1.113730.3.1.34', # ref
  ]
)

syntax_registry.register(
  Certificate.oid,[
    '2.5.4.36', # userCertificate
    '2.5.4.37', # cACertificate
    'userCertificate','userCertificate;binary',
    'cACertificate','cACertificate;binary',
  ]
)

syntax_registry.register(
  CertificateRevocationList.oid,[
    '2.5.4.38', # authorityRevocationList
    '2.5.4.39', # certificateRevocationList
    '2.5.4.53', # deltaRevocationList
    'authorityRevocationList','authorityRevocationList;binary',
    'certificateRevocationList','certificateRevocationList;binary',
    'deltaRevocationList','deltaRevocationList;binary',
  ]
)

syntax_registry.register(
  UTCTime.oid,[
    '2.5.18.1', # createTimestamp
    '2.5.18.2', # modifyTimestamp
    'createtimestamp-oid', # createtimestamp on Netscape DS 4.x
    'modifytimestamp-oid', # modifytimestamp on Netscape DS 4.x
  ]
)

